#include <R.h>
#include <Rmath.h>
#include <R_ext/Applic.h>
#include <stdlib.h>

#define MAT2(mat, i, j, nI) (mat[i+j*nI])
// auxiliar functions  for cond_cens

// poor man's uniform integer sampling between i1 and i2
int my_sam(int i1, int i2)
{ double u, umin;
  int num, j, memo;

  num = i2 - i1 +1;
  umin = 2.0;
  memo = 0;
  for (j=0; j<num; j++)
     {
       u = runif(0.0,1.0);
       if(u<umin)
	 { umin = u;
	   memo = j;
          }
     }
   return(memo + i1);
}


 
/* performs  loop for conditional sampling */ 
void adapt_cens(double *f, double *rs, int *dimData, double *out)
{
  int i, n, randx, j;
  double rowsel;
  
  GetRNGstate();
  
  n = *dimData -1 ; 
  for(i=0; i<*dimData; i++)
    out[i] = rs[i];
  
  for(i=0; i<n; i++)
    if(f[i]==0.0) 
       {
       	randx = my_sam(i,n);
       	if(randx>i)
       	  {
       	   rowsel = out[randx];	
           for(j=randx; j>i; j--)
       	        out[j] = out[j-1];
       	   out[i] = rowsel;
       	    };
       	
       };
     

  PutRNGstate();
 }



// -1 * partial likelihoo
void par_lik(double *beta,
       	     int *dimBeta,
	     double *f,
	     int *dimData,
	     double *Z,
	     double *offset,
             double *l)
{
  double eta, expon, denom;
  int i,j,ind;
  
  denom=0.0;
  ind= *dimData-1;
  for(i=0; i<*dimData; i++)
    { eta=offset[ind];
     for (j=0; j< *dimBeta; j++)
       eta+=MAT2(Z,ind,j,*dimData) * beta[j];
     expon=exp(eta);
     denom+= expon;
     *l+= f[ind] * (eta-log(denom));
     ind-=1;
   }
}
    
    
// - partial likelihood score 
void score_lik(double *beta,
       	     int *dimBeta,
	     double *f,
	     int *dimData,
	     double *Z,
	     double *offset,
             double *s)
{
  double eta, expon, denom, Ez;
  double *numer=(double *)  R_alloc(*dimBeta, sizeof(double));
  int i,j,ind;
  
   denom=0.0;
  ind= *dimData-1;
  for(j=0; j< *dimBeta; j++)
    numer[j]=0;
  for(i=0; i<*dimData; i++)
    { eta=offset[ind];
     for (j=0; j< *dimBeta; j++)
       eta+=MAT2(Z,ind,j,*dimData) * beta[j];
     expon=exp(eta);
     denom+= expon;
     for (j=0; j< *dimBeta; j++)
      {
	numer[j]+=MAT2(Z,ind,j,*dimData) * expon;
       Ez=numer[j]/denom;
       s[j]+= f[ind] * (MAT2(Z,ind,j,*dimData)-Ez) ;
       }
       ind-=1;
   }
}


// go through the data only once, and gets p+1+p+1 = s.hat+l.hat+s.til+l.til
void likscore_lik(double *beta,
		  double *betaH,
       	     int *dimBeta,
	     double *f,
	     int *dimData,
	     double *Z,
	     double *sl)
{
  double eta, etaH, expon, exponH, denom, denomH,  Ez, EzH, l, lH;
  int i,j,ind,k;
  double *numer=(double *)  R_alloc(*dimBeta, sizeof(double));
  double *numerH=(double *)  R_alloc(*dimBeta, sizeof(double));
   
   denom=0.0;
   denomH=0.0;
   l=0.0;
   lH=0.0;
  ind= *dimData - 1;
  for(j=0; j< *dimBeta; j++)
    { numer[j]=0.0;
      numerH[j]=0.0;
    }  
  for(i=0; i<*dimData; i++)
    { eta= 0.0 ;
      etaH= 0.0 ;
     for (j=0; j< *dimBeta; j++)
       { eta+=MAT2(Z,ind,j,*dimData) * beta[j];
	 etaH+=MAT2(Z,ind,j,*dimData) * betaH[j];}
     expon=exp(eta);
     exponH=exp(etaH);
     denom+= expon;
     denomH+= exponH;
     l+= f[ind] * (eta-log(denom));
     lH+= f[ind] * (etaH-log(denomH));
     for (j=0; j< *dimBeta; j++)
      { numer[j]+=MAT2(Z,ind,j,*dimData) * expon;
	numerH[j]+=MAT2(Z,ind,j,*dimData) * exponH;
        Ez=numer[j]/denom;
	EzH=numerH[j]/denomH;
        sl[j]+= f[ind] * (MAT2(Z,ind,j,*dimData)-Ez) ;
	k=*dimBeta + j + 1;
	sl[k]+= f[ind] * (MAT2(Z,ind,j,*dimData)-EzH) ;
       }
       ind-=1;
   }
  sl[*dimBeta] = l;
  k= 2 * *dimBeta + 1; 
  sl[k] = lH;
 }



// -observed information
void info_lik(double *beta,
       	     int *dimBeta,
	     double *f,
	     int *dimData,
	     double *Z,
	      double *offset,
             double *H)
{
  double eta, expon, denom, Ez2;
  int i,j,h,ind;
  double *numer=(double *)  R_alloc(*dimBeta, sizeof(double));
  double *Ez=(double *)  R_alloc(*dimBeta, sizeof(double));
  double *numer2=(double *)  R_alloc(*dimBeta * *dimBeta, sizeof(double));

  denom=0.0;
  for(j=0; j< *dimBeta; j++)
    { numer[j]=0;
      for (h=0; h< *dimBeta; h++)
	MAT2(numer2,j,h,*dimBeta)=0.0;
      }
	  
  ind= *dimData-1;	
  for(i=0; i<*dimData; i++)
    { eta=offset[ind];
     for (j=0; j< *dimBeta; j++)
       eta+=MAT2(Z,ind,j,*dimData) * beta[j];
     expon=exp(eta);
     denom+= expon;
     for (j=0; j< *dimBeta; j++)
      {
       numer[j]+=MAT2(Z,ind,j,*dimData) * expon;
       Ez[j]=numer[j]/denom;
      } 
     for (j=0; j< *dimBeta; j++)
       for (h=0; h< *dimBeta; h++)
	{ MAT2(numer2,j,h,*dimBeta)+=MAT2(Z,ind,j,*dimData) * MAT2(Z,ind,h,*dimData) * expon;
	  Ez2=MAT2(numer2,j,h,*dimBeta) / denom;
          MAT2(H,j,h,*dimBeta)+= f[ind] * (Ez2-Ez[j] * Ez[h]) ;
	 }
     ind-=1;
   }
 }



